// Variables
let body;
let buttonInterval = null;
let hoursSeparator;
let maxTime = parseInt(localStorage.getItem('ctfm-maxTime')) || 2700;
let messageTimeContainer = null;
let msSeparator;
let runTimer = true;
let timeInterval = null;
let timerContainer;
let timerHours;
let timerMinutes;
let timerSeconds;
let timerTools = null;
let toggleTimerButton = null;
let totalSeconds = 0;

let showRedTimer = getBoolean(localStorage.getItem('ctfm-showRedTimer'));

const messageSeparator = document.createElement('hr');
const separator = document.createElement('hr');
const T = Translate();
const timerOff = '<p>OFF</p>';

/**
 * Create the HTML for the timer.
 */
function createTimerHtml() {
  timerHours = document.createElement('span');
  timerHours.id = 'ctfm-hours';
  timerMinutes = document.createElement('span');
  timerMinutes.id = 'ctfm-minutes';
  timerSeconds = document.createElement('span');
  timerSeconds.id = 'ctfm-seconds';

  hoursSeparator = document.createElement('span');
  hoursSeparator.id = 'ctfm-hours-separator';
  msSeparator = document.createElement('span');
  msSeparator.id = 'ctfm-ms-separator';

  msSeparator.innerHTML = ':';
  hoursSeparator.innerHTML = ':';

  timerSeconds.innerHTML = '00';
  timerMinutes.innerHTML = '00';
}

/**
 * Add the timer HTML to the timerContainer.
 */
function showTimerHtml() {
  timerContainer = document.createElement('p');

  createTimerHtml();

  timerContainer.append(timerMinutes);
  timerContainer.append(msSeparator);
  timerContainer.append(timerSeconds);
}

/**
 * Create an interval to search an specific HTML element.
 * If that element exist, we are in the first screen and we don't have to show the timer yet.
 * If that element doesn't exist, we are in the correct page, so we add the timer to the body
 * and then initialize the timer.
 */
function showTime() {
  buttonInterval = setInterval(function () {
    // Find the UI element that we need to identify the page.
    // If it exist we haven't entered the meeting yet and will try again later.
    const findVideo = document.querySelector('[data-show-instructions]');

    if (!findVideo) {
      showTimerHtml();

      // Add our timer to the body.
      toggleTimerButton = document.createElement('div');
      toggleTimerButton.classList.add('ctfm-timer');
      toggleTimerButton.style.display = 'flex';
      toggleTimerButton.append(timerContainer);
      body = document.querySelector('body');

      body.append(toggleTimerButton);

      startTime();
    }
  }, 250);
}

/**
 * Initialize the timer.
 */
function startTime() {
  if (buttonInterval) {
    clearInterval(buttonInterval);
  }

  // Show the timer tools on hover.
  showTimerTools();

  // Set the timer draggable.
  setTimerDraggable(body);

  totalSeconds = 0;

  let callIsFinished;

  // Initialize the timer interval to increase the counting.
  timeInterval = setInterval(function() {
    callIsFinished = document.querySelector('[data-user-identifier]');

    // If callIsFinished exist, show the total time on the last Google Meet page.
    if (callIsFinished) {
      callFinished(callIsFinished);
    }

    totalSeconds++;

    // If the user set to show the timer in red
    // and the time is equal to the totalSeconds, mark the timer in red.
    if (showRedTimer) {
      if (totalSeconds === maxTime) {
        makeCallRed();
      }
    }

    printTime();
  }, 1000);
}

/**
 * Add a class to the timer to make it red.
 */
function makeCallRed() {
  toggleTimerButton.classList.add('ctfm-call-red');
  showMessage();
}

/**
 * If the call finished, remove the timer and clear the interval.
 * Then show the total time that the user has been on the call on the last screen of Google Meet.
 * @param callIsFinished
 */
function callFinished(callIsFinished) {
  removeTimer();
  clearInterval(timeInterval);

  const callFinishedText = document.createElement('p');
  callFinishedText.classList.add('ctfm-call-finished');

  const seconds = pad(totalSeconds % 60);
  const minutes = pad(parseInt(totalSeconds / 60) % 60);
  const hours = parseInt(totalSeconds / 3600);

  let totalTime;

  if (totalSeconds < 3600) {
    totalTime = `${minutes}:${seconds}`;
  } else {
    totalTime = `${hours}:${minutes}:${seconds}`;
  }

  // If the time if different of zero, add it to an specific HTML element from the last page.
  if (seconds !== '00' || minutes !== '00' || hours !== 0) {
    callFinishedText.innerHTML = `<span>${T('totalTime')}</span> ${totalTime}`;
    callIsFinished.children[0].children[0].append(callFinishedText);
  }
}

/**
 * Show the message that the time was exceeded on the timer tools.
 */
function showMessage() {
  messageTimeContainer = document.createElement('div');
  messageTimeContainer.classList.add('ctfm-timer-message');

  const messageTimeExceeded = document.createElement('p');
  messageTimeExceeded.innerHTML = T('timeExceededMessage');

  messageTimeContainer.append(messageTimeExceeded);

  timerTools.insertBefore(messageSeparator, timerTools.firstChild);
  timerTools.insertBefore(messageTimeContainer, timerTools.firstChild);
}

/**
 * Hide the message that the time was exceeded on the timer tools.
 */
function hideMessage() {
  if (messageTimeContainer) {
    timerTools.removeChild(messageTimeContainer);
    timerTools.removeChild(messageSeparator);

    toggleTimerButton.classList.remove('ctfm-call-red');
    messageTimeContainer = null;
  }
}

/**
 * Print the time every second.
 * If the totalSeconds is higher or equal to 3600 (an hour)
 * add the hour time.
 */
function printTime() {
  timerSeconds.innerHTML = pad(totalSeconds % 60);
  timerMinutes.innerHTML = pad(parseInt(totalSeconds / 60) % 60);

  if (totalSeconds >= 3600 && totalSeconds % 3600 === 0) {
    hoursSeparator.innerHTML = ':';
    timerHours.innerHTML = parseInt(totalSeconds / 3600);

    timerContainer.prepend(hoursSeparator);
    timerContainer.prepend(timerHours);

    toggleTimerButton.classList.add('ctfm-timer-hour');

    // Make the hour element draggable.
    setHourDraggable();
  }
}

/**
 * Add the timer tools to the hover of the timer.
 */
function showTimerTools() {
  // Add timer tools
  timerTools = document.createElement('div');
  timerTools.classList.add('ctfm-timer-tools');
  timerTools.onclick = e => e.stopPropagation();
  toggleTimerButton.appendChild(timerTools);

  // Add input with the max time to make it red
  addRedTimerCheckbox();
  addMaxTimeInput();
  addTimerToolsButtons();
}

/**
 * Add the checkbox to the timer tools.
 * This checkbox allows the user to show or hide the time exceed message.
 */
function addRedTimerCheckbox() {
  const redTimerContainer = document.createElement('div');
  redTimerContainer.classList.add('ctfm-red-timer');

  // Create the message label.
  const redTimerLabel = document.createElement('label');
  redTimerLabel.classList.add('ctfm-red-timer-checkbox-label');
  redTimerLabel.innerHTML = T('redTimer');
  redTimerLabel.setAttribute('for', 'ctfm-red-timer-checkbox');

  // Create the checkbox.
  const redTimerInput = document.createElement('input');
  redTimerInput.type = 'checkbox';
  redTimerInput.checked = showRedTimer;
  redTimerInput.id = 'ctfm-red-timer-checkbox';
  redTimerInput.classList.add('ctfm-red-timer-checkbox');

  // If the checkbox is clicked.
  redTimerInput.onchange = e => {
    showRedTimer = e.target.checked;
    localStorage.setItem('ctfm-showRedTimer', e.target.checked);

    // If the user set to show the timer in red
    // and the time is equal or higher to the totalSeconds, mark the timer in red.
    if (showRedTimer) {
      if (totalSeconds >= maxTime) {
        makeCallRed();
      }

      // If the showRedTimer is true, make the checkbox enabled.
      toggleDisabledInput('ctfm-max-timer-input', false);
    } else {
      // If not set the showRedTimer, hide the message and make the checkbox disabled.
      hideMessage();
      toggleDisabledInput('ctfm-max-timer-input', true);
    }
  };

  redTimerContainer.append(redTimerInput);
  redTimerContainer.append(redTimerLabel);

  timerTools.appendChild(redTimerContainer);
}

/**
 * Add the input to set the max time to make the timer red.
 * This input allows the user to set a specific time to show the exceed message.
 */
function addMaxTimeInput() {
  const maxTimerContainer = document.createElement('div');
  maxTimerContainer.classList.add('ctfm-max-timer');

  // Create the label.
  const maxTimerLabel = document.createElement('label');
  maxTimerLabel.classList.add('ctfm-max-timer-input-label');
  maxTimerLabel.innerHTML = T('timeMaxInput');
  maxTimerLabel.setAttribute('for', 'ctfm-max-timer-input');

  // Create the input.
  const maxTimerInput = document.createElement('input');
  maxTimerInput.type = 'text';
  maxTimerInput.classList.add('ctfm-max-timer-input');
  maxTimerInput.id = 'ctfm-max-timer-input';
  maxTimerInput.value = parseInt(maxTime / 60);

  // If the user typed on the input
  maxTimerInput.onkeyup = event => {
    // If the input has value.
    if (event.target.value) {
      maxTime = parseInt(event.target.value) * 60;
      localStorage.setItem('ctfm-maxTime', maxTime);

      // Hide the exceeded time message.
      hideMessage();

      // If the user set to show the timer in red
      // and the time is equal or higher to the totalSeconds, mark the timer in red.
      if (showRedTimer) {
        if (totalSeconds >= maxTime) {
          makeCallRed();
        }
      }
    }
  };

  // Allows the user to only use numbers.
  maxTimerInput.onkeypress = event => {
    return isNumberKey(event);
  };

  // Add the help text under the input.
  const userHelp = document.createElement('span');
  userHelp.classList.add('ctfm-timer-tools-helper');
  userHelp.innerHTML = T('timeMaxHelp');

  maxTimerContainer.append(maxTimerLabel);
  maxTimerContainer.append(maxTimerInput);
  maxTimerContainer.append(userHelp);

  timerTools.appendChild(separator.cloneNode());
  timerTools.appendChild(maxTimerContainer);

  // If the user set NOT to show the timer in red, make the input disabled.
  if (!showRedTimer) {
    toggleDisabledInput('ctfm-max-timer-input', true);
  }
}

/**
 * Enable or Disable an input.
 * @param elementId
 * @param disabled
 */
function toggleDisabledInput(elementId, disabled) {
  document.getElementById(elementId).disabled = disabled;
}

/**
 * Allow to the user to use only numbers.
 * @param event
 * @returns {boolean}
 */
function isNumberKey(event) {
  const isNumber = event.key.match('^[0-9]');

  return !!isNumber;
}

/**
 * Add a zero in front to a number with one character.
 * @param value
 * @returns {string}
 */
function pad(value) {
  return ('0' + value).slice(-2);
}

/**
 * Make the timer disabled.
 */
function disableTimer() {
  runTimer = null;

  toggleTimerButton.classList.remove('ctfm-call-red');
  hideMessage();

  timerContainer.innerHTML = timerOff;

  if (timeInterval) {
    clearInterval(timeInterval);
  }
}

/**
 * Make the timer disabled.
 */
function enableTimer() {
  runTimer = true;

  toggleTimerButton.innerHTML = '';

  showTimerHtml();
  toggleTimerButton.append(timerContainer);

  startTime();
}

/**
 * Remove the timer from the HTML.
 */
function removeTimer() {
  toggleTimerButton.remove();
}

/**
 * Check if the value exist or not.
 * @param value
 * @returns {boolean}
 */
function getBoolean(value) {
  if (value == null) {
    return true;
  }

  return value === true || value === "true";
}

/**
 * Start the timer once the window load.
 */
function addListeners() {
  document.body.append(timerStyles());

  showTime();
}

//on init perform based on chrome storage value
window.onload = function() {
  addListeners();
};
