/**
 * Add the buttons to the timer tools.
 */
function addTimerToolsButtons() {
  const timerToolsButtonsContainer = document.createElement('div');
  timerToolsButtonsContainer.classList.add('ctfm-timer-tools-buttons');

  const timerButtons = document.createElement('div');
  timerButtons.classList.add('ctfm-timer-buttons');

  const positionButtons = document.createElement('div');
  positionButtons.classList.add('ctfm-position-buttons');

  const resetButton = resetTimerButton();
  const turnOnOffButton = turnOnOfButton(resetButton);
  const resetPositionButton = resetTimerPositionButton();

  timerButtons.append(turnOnOffButton);
  timerButtons.append(resetButton);
  timerToolsButtonsContainer.append(timerButtons);

  positionButtons.append(resetPositionButton);
  timerToolsButtonsContainer.append(positionButtons);

  const turnOffSeparator = separator.cloneNode();
  turnOffSeparator.classList.add('ctfm-timer-tools-separator');

  timerTools.appendChild(turnOffSeparator);
  timerTools.appendChild(timerToolsButtonsContainer);
}

/**
 * Turn on/off timer button.
 */
function turnOnOfButton(resetButton) {
  const turnOnOffButton = document.createElement('button');
  turnOnOffButton.classList.add('ctfm-turn-on-off-btn');
  turnOnOffButton.classList.add('ctfm-turn-off');
  turnOnOffButton.innerHTML = T('turnOff');

  turnOnOffButton.onclick = () => {
    if (runTimer) {
      disableTimer();
      toggleDisabledInput('ctfm-max-timer-input', true);
      toggleDisabledInput('ctfm-red-timer-checkbox', true);

      document.getElementById('ctfm-reset').disabled = true;
      resetButton.classList.add('ctfm-reset-disabled');

      turnOnOffButton.innerHTML = T('turnOn');
      turnOnOffButton.classList.add('ctfm-turn-on');
    } else {
      enableTimer();
      toggleDisabledInput('ctfm-max-timer-input', false);
      toggleDisabledInput('ctfm-red-timer-checkbox', false);

      document.getElementById('ctfm-reset').disabled = false;
      resetButton.classList.remove('ctfm-reset-disabled');

      turnOnOffButton.innerHTML = T('turnOff');
      turnOnOffButton.classList.add('ctfm-turn-off');
    }
  };

  return turnOnOffButton;
}

/**
 * Reset timer button.
 */
function resetTimerButton() {
  const resetButton = document.createElement('button');
  resetButton.classList.add('ctfm-reset-btn');
  resetButton.classList.add('ctfm-reset');
  resetButton.id= 'ctfm-reset';
  resetButton.innerHTML = T('reset');

  resetButton.onclick = () => {
    disableTimer();
    toggleDisabledInput('ctfm-max-timer-input', true);
    toggleDisabledInput('ctfm-red-timer-checkbox', true);

    enableTimer();
    toggleDisabledInput('ctfm-max-timer-input', false);
    toggleDisabledInput('ctfm-red-timer-checkbox', false);
  };

  return resetButton;
}

/**
 * Reset timer position.
 */
function resetTimerPositionButton() {
  const resetPositionButton = document.createElement('button');
  resetPositionButton.classList.add('ctfm-reset-position-btn');
  resetPositionButton.classList.add('ctfm-reset-position');
  resetPositionButton.id= 'ctfm-reset-position';
  resetPositionButton.innerHTML = T('resetPosition');

  resetPositionButton.onclick = () => {
    resetTimerPosition();
  };

  return resetPositionButton;
}
