// Draggable items
let dragItem;
let dragItemHours;
let dragItemHoursSeparator;
let dragItemMinutes;
let dragItemMsSeparator;
let dragItemSeconds;
let container;

// Drag element variables
let active = false;
let currentX;
let currentY;
let initialX;
let initialY;
let xOffset = 0;
let yOffset = 0;

window.onresize = resetTimerPosition;

/**
 * Configure the entire variables to drag the timer
 * @param body
 */
function setTimerDraggable(body) {
  dragItem = document.querySelector(".ctfm-timer");
  dragItemMinutes = document.querySelector("#ctfm-minutes");
  dragItemMsSeparator = document.querySelector("#ctfm-ms-separator");
  dragItemSeconds = document.querySelector("#ctfm-seconds");
  container = body;

  container.addEventListener("touchstart", dragStart, false);
  container.addEventListener("touchend", dragEnd, false);
  container.addEventListener("touchmove", drag, false);

  container.addEventListener("mousedown", dragStart, false);
  container.addEventListener("mouseup", dragEnd, false);
  container.addEventListener("mousemove", drag, false);

  setFirstPosition();
}

/**
 * Set the first position for the timer.
 * If it has a saved position, set that positions.
 * If it hasn't a saved position, set the 0,0 position.
 */
function setFirstPosition() {
  const height = document.documentElement.clientHeight;
  const width = document.documentElement.clientWidth;

  const initialPosition = getTimerPosition();

  if (initialPosition && initialPosition.xPos < width && initialPosition.yPos < height) {
    initialX = initialPosition.xPos;
    initialY = initialPosition.yPos;

    xOffset = initialPosition.xPos;
    yOffset = initialPosition.yPos;

    currentX = initialPosition.xPos;
    currentY = initialPosition.yPos;

    setTranslate(currentX, currentY, dragItem);
    setTimerToolsPosition(currentX, currentY);
  }
}

/**
 * Set the hour drag item.
 */
function setHourDraggable() {
  dragItemHours = document.querySelector("#ctfm-hours");
  dragItemHoursSeparator = document.querySelector("#ctfm-hours-separator");
}

/**
 * Execute when the drag begins.
 * @param e
 */
function dragStart(e) {
  if (e.type === "touchstart") {
    initialX = e.touches[0].clientX - xOffset;
    initialY = e.touches[0].clientY - yOffset;
  } else {
    initialX = e.clientX - xOffset;
    initialY = e.clientY - yOffset;
  }

  if (
    e.target === dragItem ||
    e.target === dragItemHours ||
    e.target === dragItemHoursSeparator ||
    e.target === dragItemMinutes ||
    e.target === dragItemMsSeparator ||
    e.target === dragItemSeconds
  ) {
    active = true;
  }
}

/**
 * Set the last position and save it on the localstorage.
 * @param e
 */
function dragEnd(e) {
  initialX = currentX;
  initialY = currentY;

  active = false;

  setTimerPosition(currentX, currentY);
}

/**
 * Allow to drag the timer.
 * @param e
 */
function drag(e) {
  if (active) {

    e.preventDefault();

    if (e.type === "touchmove") {
      currentX = e.touches[0].clientX - initialX;
      currentY = e.touches[0].clientY - initialY;
    } else {
      currentX = e.clientX - initialX;
      currentY = e.clientY - initialY;
    }

    xOffset = currentX;
    yOffset = currentY;

    setTranslate(currentX, currentY, dragItem);
    setTimerToolsPosition(currentX, currentY);
  }
}

/**
 * Allow to move the timer inside of the box of the page.
 * @param xPos
 * @param yPos
 * @param el
 */
function setTranslate(xPos, yPos, el) {
  const height = document.documentElement.clientHeight;
  const width = document.documentElement.clientWidth;

  let xPosTimer = xPos;
  let yPosTimer = yPos + 50;

  if (document.getElementsByClassName('ctfm-timer ctfm-timer-hour').length) {
    xPosTimer += 70;
  } else {
    xPosTimer += 50;
  }

  if (xPos === 0 && yPos === 0) {
    el.style.transform = "translate3d(" + xPos + "px, " + yPos + "px, 0)";
    return false;
  }

  if (xPos <= 0 || xPosTimer >= width) {
    el.style.transform = "translate3d(" + yPos + "px, 0)";
    return false;
  }

  if (yPos <= 0 || yPosTimer >= height) {
    el.style.transform = "translate3d(" + xPos + "px, 0)";
    return false;
  }

  el.style.transform = "translate3d(" + xPos + "px, " + yPos + "px, 0)";
  return false;
}

/**
 * Save the timer tool position according to the timer position on the page.
 * @param xPos
 * @param yPos
 */
function setTimerToolsPosition(xPos, yPos) {
  const timerTools = document.querySelector('.ctfm-timer-tools');

  if (timerTools) {
    const halfScreenHeight = window.screen.availHeight / 2;
    const halfScreenWidth = window.screen.availWidth / 2;

    if (yPos > halfScreenHeight) {
      timerTools.classList.add('ctfm-timer-tools-bottom');
    } else {
      if (timerTools && timerTools.classList.contains('ctfm-timer-tools-bottom')) {
        timerTools.classList.remove('ctfm-timer-tools-bottom');
      }
    }

    if (xPos > halfScreenWidth) {
      timerTools.classList.add('ctfm-timer-tools-right');
    } else {
      if (timerTools && timerTools.classList.contains('ctfm-timer-tools-right')) {
        timerTools.classList.remove('ctfm-timer-tools-right');
      }
    }
  }
}

/**
 * Set the time position on the local storage
 * @param xPos
 * @param yPos
 */
function setTimerPosition(xPos, yPos) {
  const timerPosition = JSON.stringify({ xPos, yPos });
  localStorage.setItem('ctfm-timerPosition', timerPosition);
}

/**
 * Get the time position saved on the local storage.
 */
function getTimerPosition() {
  const timerPosition = localStorage.getItem('ctfm-timerPosition');
  return JSON.parse(timerPosition);
}

/**
 * Reset timer to the original position X: 0 and Y: 0.
 */
function resetTimerPosition() {
  initialX = 0;
  initialY = 0;

  xOffset = 0;
  yOffset = 0;

  currentX = 0;
  currentY = 0;

  setTranslate(currentX, currentY, dragItem);
  setTimerToolsPosition(currentX, currentY);
  setTimerPosition(currentX, currentY);
}
